Clazz.declarePackage ("J.adapter.readers.xml");
Clazz.load (["J.adapter.readers.xml.XmlCmlReader"], "J.adapter.readers.xml.XmlMOReader", ["java.util.Hashtable", "JU.AU", "$.Lst", "$.PT", "J.adapter.readers.quantum.MOReader", "J.adapter.smarter.Resolver", "JU.Logger"], function () {
c$ = Clazz.decorateAsClass (function () {
this.moReader = null;
this.skipMOs = false;
this.htSlaterIDs = null;
this.htAtomBases = null;
this.basisExp = null;
this.basisCoef = null;
this.basisCoefSP = null;
this.basisId = null;
this.isSpherical = false;
this.minL = 0;
this.maxL = 0;
this.basisIds = null;
this.basisAtoms = null;
this.orbOcc = 0;
this.orbEnergy = 0;
this.gaussianCount = 0;
this.slaterCount = 0;
this.lstGaussians = null;
this.moCount = 0;
Clazz.instantialize (this, arguments);
}, J.adapter.readers.xml, "XmlMOReader", J.adapter.readers.xml.XmlCmlReader);
Clazz.defineMethod (c$, "processStartMO", 
function (localName) {
if (localName.equals ("basisset")) {
if (this.moReader == null && !this.skipMOs) {
var rdr = J.adapter.smarter.Resolver.getReader ("MO", this.parent.htParams);
if ((Clazz.instanceOf (rdr, String))) {
this.skipMOs = true;
} else {
this.moReader = rdr;
this.moReader.asc = this.asc;
this.lstGaussians =  new JU.Lst ();
this.htAtomBases =  new java.util.Hashtable ();
this.htSlaterIDs =  new java.util.Hashtable ();
}}return true;
}if (this.moReader != null) {
if (localName.equals ("basisgroup")) {
this.basisId = this.atts.get ("id");
this.isSpherical = "spherical".equalsIgnoreCase (this.atts.get ("angular"));
this.minL = JU.PT.parseInt (this.atts.get ("minl"));
this.maxL = JU.PT.parseInt (this.atts.get ("maxl"));
return true;
}if (localName.equals ("basisexponents") || localName.equals ("basiscontraction")) {
this.setKeepChars (true);
return true;
}if (localName.equals ("orbital") && this.gaussianCount > 0) {
this.orbOcc = JU.PT.parseFloat (this.atts.get ("occupation"));
this.orbEnergy = JU.PT.parseFloat (this.atts.get ("energy"));
this.setKeepChars (true);
return true;
}}return false;
}, "~S");
Clazz.defineMethod (c$, "processEndMO", 
function (localName) {
if (this.moReader != null) {
if (localName.equals ("basisset")) {
this.buildSlaters ();
return true;
}if (localName.equals ("basisgroup")) {
var otype;
switch (this.minL) {
case 0:
otype = (this.maxL == 1 ? "L" : "S");
break;
case 1:
otype = "P";
break;
default:
otype = (this.minL <= 7 ? "SPDFGHI".substring (this.minL, this.minL + 1) : "?");
if (this.isSpherical) otype = (2 * (this.minL) + 1) + otype;
}
this.lstGaussians.addLast ( Clazz.newArray (-1, [this.basisExp, this.basisCoef, this.basisCoefSP]));
this.htSlaterIDs.put (this.basisId,  Clazz.newIntArray (-1, [-1, this.moReader.getQuantumShellTagID (otype), this.gaussianCount, this.basisExp.length]));
this.gaussianCount += this.basisExp.length;
return true;
}if (localName.equals ("basisexponents")) {
this.basisExp = JU.PT.parseFloatArray (this.chars);
this.setKeepChars (false);
return true;
}if (localName.equals ("basiscontraction")) {
var a = JU.PT.parseFloatArray (this.chars);
if ("1".equals (this.atts.get ("id"))) {
this.basisCoef = a;
this.basisCoefSP = null;
} else {
this.basisCoefSP = a;
}this.setKeepChars (false);
return true;
}if (localName.equals ("orbital") && this.gaussianCount > 0) {
this.moCount = this.moReader.addCoef ( new java.util.Hashtable (), JU.PT.parseFloatArray (this.chars), null, this.orbEnergy, this.orbOcc, this.moCount);
this.setKeepChars (false);
return true;
}if (localName.equals ("orbitals")) {
this.moReader.setMOData (true);
return true;
}if (this.state == 19) {
if (localName.equals ("bases")) {
this.basisIds = this.getXlink (this.atts.get ("href"), "basisGroup", 0);
} else if (localName.equals ("atoms")) {
this.basisAtoms = this.getXlink (this.atts.get ("href"), "atom", 1);
} else if (localName.equals ("association")) {
for (var i = this.basisAtoms.length; --i >= 0; ) {
var a = this.asc.getAtomFromName (this.basisAtoms[i]);
if (a == null) {
JU.Logger.error ("XmlMolproReader atom not found; orbitals skipped: " + a);
this.moReader = null;
break;
}this.htAtomBases.put (Integer.$valueOf (a.index), this.basisIds);
}
this.slaterCount += this.basisIds.length * this.basisAtoms.length;
}return true;
}}return false;
}, "~S");
Clazz.defineMethod (c$, "buildSlaters", 
 function () {
var gaussians = JU.AU.newFloat2 (this.gaussianCount);
for (var i = 0, p = 0, n = this.lstGaussians.size (); i < n; i++) {
var a = this.lstGaussians.get (i);
var exp = a[0];
var coef = a[1];
var coefSP = a[2];
for (var j = 0; j < exp.length; j++) gaussians[p++] =  Clazz.newFloatArray (-1, [exp[j], coef[j], coefSP == null ? 0 : coefSP[j]]);

}
this.moReader.gaussians = gaussians;
var slaters =  new JU.Lst ();
for (var i = this.asc.getAtomSetAtomIndex (this.asc.iSet), n = i + this.asc.getAtomSetAtomCount (this.asc.iSet); i < n; i++) {
var ids = this.htAtomBases.get (Integer.$valueOf (this.asc.atoms[i].index));
if (ids == null) continue;
for (var k = 0; k < ids.length; k++) {
var slater = JU.AU.arrayCopyI (this.htSlaterIDs.get (ids[k]), -1);
this.moReader.shells = slaters;
slater[0] = i;
slaters.addLast (slater);
}
}
JU.Logger.info ("XmlMolproReader created " + this.gaussianCount + " gaussians and " + this.slaterCount + " slaters");
});
Clazz.defineMethod (c$, "getXlink", 
 function (href, key, apt) {
var tokens = JU.PT.split (href.substring (href.indexOf (key + "[")), "'");
var data =  new Array (Clazz.doubleToInt (tokens.length / 2));
for (var i = 1, pt = 0; i < tokens.length; i += 2) data[pt++] = tokens[i].substring (apt);

return data;
}, "~S,~S,~N");
});
